@@ -1,6 +1,6 @@
 use crate::process::ProcessInfo;
 use crate::util::bytify;
-use crate::{column_default, Column};
+use crate::{Column, column_default};
 use std::cmp;
 use std::collections::HashMap;
 
@@ -16,7 +16,7 @@ impl ReadBytes {
     pub fn new(header: Option<String>) -> Self {
         let header = header.unwrap_or_else(|| String::from("Read"));
         let unit = String::from("[B/s]");
-        ReadBytes {
+        Self {
             fmt_contents: HashMap::new(),
             raw_contents: HashMap::new(),
             width: 0,
@@ -29,12 +29,11 @@ impl ReadBytes {
 #[cfg(any(target_os = "linux", target_os = "android"))]
 impl Column for ReadBytes {
     fn add(&mut self, proc: &ProcessInfo) {
-        let (fmt_content, raw_content) = if proc.curr_io.is_some() && proc.prev_io.is_some() {
+        let (fmt_content, raw_content) = if let Some(curr_io) = proc.curr_io
+            && let Some(prev_io) = proc.prev_io
+        {
             let interval_ms = proc.interval.as_secs() + u64::from(proc.interval.subsec_millis());
-            let io = (proc.curr_io.as_ref().unwrap().read_bytes
-                - proc.prev_io.as_ref().unwrap().read_bytes)
-                * 1000
-                / interval_ms;
+            let io = (curr_io.read_bytes - prev_io.read_bytes) * 1000 / interval_ms;
             (bytify(io), io)
         } else {
             (String::new(), 0)
@@ -44,7 +43,7 @@ impl Column for ReadBytes {
         self.raw_contents.insert(proc.pid, raw_content);
     }
 
-    column_default!(u64);
+    column_default!(u64, true);
 }
 
 #[cfg(target_os = "macos")]
@@ -65,7 +64,7 @@ impl Column for ReadBytes {
         self.raw_contents.insert(proc.pid, raw_content);
     }
 
-    column_default!(u64);
+    column_default!(u64, true);
 }
 
 #[cfg(target_os = "windows")]
@@ -81,7 +80,7 @@ impl Column for ReadBytes {
         self.raw_contents.insert(proc.pid, raw_content);
     }
 
-    column_default!(u64);
+    column_default!(u64, true);
 }
 
 #[cfg(target_os = "freebsd")]
@@ -103,5 +102,5 @@ impl Column for ReadBytes {
         self.raw_contents.insert(proc.pid, raw_content);
     }
 
-    column_default!(u64);
+    column_default!(u64, true);
 }
